#This code is 
#   to plot results of Random Forest analysis 
#   to calculate p-values
#as described in Ahmad et al. 2018

#It requires all the files present in RnadomForestAnalysisRequiredOutputFiles.zip to be
#present in the same folder.


get_SW_PVals<-function(randRuns){
  ## It return Shapiro-Wilk Normality Test p-value
  randR = t(randRuns)
  res = c()
  for(i in 1:13){
    res = c(res,shapiro.test(randR[,i])$p.value)
  }
  return(res)
}


getPVals<-function(randRuns,rfObs){
  imp = rfObs
  randR = t(randRuns)
  mean.norm = apply(randR, 2, mean)
  stdev.norm = apply(randR, 2, sd)
  res <- c()
  for(i in 1:length(imp)){
    #calculate 'upper' pvalue
    p.val <- pnorm(imp[i], mean.norm[i], stdev.norm[i], lower.tail=F)

    res <- c(res, p.val)
  }
  return(res)
}
cis <-function(x){
  return(t.test(x,conf.level = 0.99)$conf.int)
}




setEPS(width=6.64,height=5.2)
postscript("RFplots_final2017.eps")
par(mfrow=c(2,3),mar=c(4,4,0,0))


###Back purging files

fx1 = "Top120SNPs_RPL_Comp_corx_Var_explained_10000trees.txt"
fx2 = "Top120SNPs_FL_corx_Var_explained_10000trees.txt"

out120_f1<-read.table(fx1)
out120_f2<-read.table(fx2)

##HEME
f3 = "RF_20000trees_HEME_Comp_corx_FINAL.txt"
f3_rands = "RF_20000trees_HEME_Comp_corx_random_allnIterations_FINAL.txt"
out20000 = read.table(f3,head=T)
out20000_rands = read.table(f3_rands,head=T)

p1val = getPVals(randRuns=out20000_rands[,2:51],rfObs=out20000$rf3_v1)
pHeme_sw_val = get_SW_PVals(randRuns=out20000_rands[,2:51])
plot(NULL,NULL,xlim=c(0,10),ylim=c(-20,60),
     ylab = "Percentage of variation explained",
     xlab = "",
     main = NULL,axes=FALSE)
for (i in 2:51){
  points(log(out20000_rands$nSNPs),out20000_rands[,i],pch = 19,col="grey",type = "l")
}
points(log(out20000$nSNPs),out20000$rf3_v1,pch = 19,type = "l",lwd=4)
points(log(out20000$nSNPs),out20000$rf3_v1,pch  = "|",type = "p",cex=2)
text(log(out20000$nSNPs), y = out20000$rf3_v1+12, labels = format(signif(p1val,digits = 2)),cex= 1, pos = 3,srt=90)

#title("Hematocrit",line = -1.5,adj = 1)
text(4.8,-20,"Hematocrit")
axis(2)
axis(1,at = log(out20000$nSNPs),labels = out20000$nSNPs,las=3)



par(mar=c(4,4,0,0)+0.1, new=F)
##RPL
f1 = "RF_20000trees_RPL_Comp_corx_FINAL.txt"
f1_rands = "RF_20000trees_RPL_Comp_corx_random_allnIterations_FINAL.txt"
out20000 = read.table(f1,head=T)
out20000_rands = read.table(f1_rands,head=T)
pRPL_sw_val = get_SW_PVals(randRuns=out20000_rands[,2:51])
p1val = getPVals(randRuns=out20000_rands[,2:51],rfObs=out20000$rf3_v1)
plot(NULL,NULL,xlim=c(0,10),ylim=c(-20,60),axes=FALSE,
     ylab = "",
     xlab = "",
     main = NULL)
for (i in 2:51){
  points(log(out20000_rands$nSNPs),out20000_rands[,i],pch = 19,col="grey",type = "l")
}
points(log(out20000$nSNPs),out20000$rf3_v1,pch = 19,type = "l",lwd=4)
points(log(out20000$nSNPs),out20000$rf3_v1,pch  = "|",type = "p",cex=2)
text(log(out20000$nSNPs), y = out20000$rf3_v1+10, labels = format(signif(p1val,digits = 2)),cex= 1, pos = 3,srt=90)
#title("RPL",line = -1.5,adj = 1)
text(4.8,-20,"RPL")
axis(2)
axis(1,at = log(out20000$nSNPs),labels = out20000$nSNPs,las=3)


par(mar=c(4,4,0,0)+0.1, new=F)
#RPL back purge
plot(NULL,NULL,xlim=c(2,120),ylim=c(0.0,50),axes=F,
     ylab = "Percentage of variation explained",
     xlab = "",
     main = "")
#title("RPL",line = -1.5,adj = 1)
text(60,10,"RPL")
points(2:120,out120_f1$Average[2:120]*100,pch = 19)
axis(1)
axis(2)
#40.6% of the variation was explained by 13 SNPs.


par(mar=c(4,4,0,0)+0.1, new=F)
##kidRat
f4 = "RF_20000trees_kidRat_Comp_corx_FINAL.txt"
f4_rands = "RF_20000trees_kidRat_Comp_corx_random_allnIterations_FINAL.txt"
out20000 = read.table(f4,head=T)
out20000_rands = read.table(f4_rands,head=T)
pKS_sw_val = get_SW_PVals(randRuns=out20000_rands[,2:51])
p1val = getPVals(randRuns=out20000_rands[,2:51],rfObs=out20000$rf3_v1)
plot(NULL,NULL,xlim=c(0,10),ylim=c(-20,60),axes=FALSE,
     ylab = "Percentage of variation explained",
     xlab = "",
     main = NULL)
for (i in 2:51){
  points(log(out20000_rands$nSNPs),out20000_rands[,i],pch = 19,col="grey",type = "l")
}
points(log(out20000$nSNPs),out20000$rf3_v1,pch = 19,type = "l",lwd=4)
points(log(out20000$nSNPs),out20000$rf3_v1,pch  = "|",type = "p",cex=2)
text(log(out20000$nSNPs), y = out20000$rf3_v1+12, labels = format(signif(p1val,digits = 2)),cex= 1, pos = 3,srt=90)

#title("Kidney Swolleness",line = -1.5,adj = 1)
text(4.8,-20,"Kidney Swolleness")
axis(2)
axis(1,at = log(out20000$nSNPs),labels = out20000$nSNPs,las=3)


par(mar=c(4,4,0,0)+0.1, new=F)
##Fork Length
f2 = "RF_20000trees_FL_corx_FINAL.txt"
f2_rands = "RF_20000trees_FL_corx_random_allnIterations_FINAL.txt"
out20000 = read.table(f2,head=T)
out20000_rands = read.table(f2_rands,head=T)
pFL_sw_val = get_SW_PVals(randRuns=out20000_rands[,2:51])
p1val = getPVals(randRuns=out20000_rands[,2:51],rfObs=out20000$rf3_v1)
plot(NULL,NULL,xlim=c(0,10),ylim=c(-20,60),axes=FALSE,
     ylab = "",
     xlab = "Number of SNPs (log scale)",
     main = "")
for (i in 2:51){
  points(log(out20000_rands$nSNPs),out20000_rands[,i],pch = 19,col="grey",type = "l")
}
points(log(out20000$nSNPs),out20000$rf3_v1,pch = 19,type = "l",lwd=4)
points(log(out20000$nSNPs),out20000$rf3_v1,pch  = "|",type = "p",cex=2)
text(log(out20000$nSNPs), y = out20000$rf3_v1+10, labels = format(signif(p1val,digits = 2)),cex= 1, pos = 3,srt=90)
text(4.8,-20,"Fork Length")
axis(1,at = log(out20000$nSNPs),labels = out20000$nSNPs,las=3)
axis(2)



par(mar=c(4,4,0,0)+0.1, new=F)
#back puriging fork length
plot(NULL,NULL,xlim=c(2,120),ylim=c(0.0,50),yaxt='n',axes = F,
     ylab = "Percentage of variation explained",
     xlab = "Number of SNPs",
     main = "")
#title("Fork length",line = -1.5,adj = 1)
text(60,10,"Fork length")
points(2:120,out120_f2$Average[2:120]*100,pch = 19,lwd = 1)
axis(1)
axis(2)
# 16 SNPs explained 34.67 % 



dev.off()

