library(randomForest)
nIterations = 50
#SNPs halved at each iteration of Coarse Sweep
nS = c(3830,1915,958,479,240,120,60,30,15,8,4,2)
out20000 = data.frame(nSNPs = c(7661,nS))
for (iter in 1:nIterations){
##read imputed SNP matrix
data = read.table("Imputed7661X255_final.txt",head=T,check.names = F)
x0 = as.matrix(data[,-1])
rownames(x0) = data[,1]
#read principal components
pz = read.table("fivePCAs.txt",head=T)
p1 = data.frame(pz[match(rownames(x0),pz[,1]),2:6])
rownames(p1) = rownames(x0)
#read phenotypes
phenos <- read.table("255_phenotypes_genabel.txt",header = T)
rownames(phenos) <- as.character(phenos$id)
phenos <- phenos[,-1]
phenos <- phenos[rownames(x0),]

#correction of SNPs using first 5 PCs
corr_x0 = matrix(0,nrow=255,ncol=7661)
rownames(corr_x0)<- rownames(x0)
colnames(corr_x0)<-colnames(x0)
for (i in 1:ncol(x0))
{
  LM_SNP_i <- glm(x0[,i] ~ .,data = p1,na.action=na.exclude)  
  corr_x0[,i] <- residuals(LM_SNP_i)
  if(i%%500==0) print(i)
}
#scale fork length
fl = scale(phenos$fork.length)
#correction of trait using first 5 PCs
FLx = glm(fl ~ .,data = p1,na.action=na.exclude)
FL_corx_random = sample(residuals(FLx))#Randomize the trait order

#fitting random forest model, trait residuals as responce and snps as feeatures
rf3_1 = randomForest(x = corr_x0, y = FL_corx_random, importance=TRUE ,proximity=TRUE, ntree=20000)
imp_all_1 = as.data.frame(importance(rf3_1))

#Refine individual SNP importance scores by deviding to absolute minimum importnace
imp_all_1$imp_score = imp_all_1[,1]/abs(min( imp_all_1[,1]))

#Write  7661 SNPs importance
write.table(imp_all_1,paste("RANDOM_RF_Files/importance_all_imp_RF3_1_SNPs_FL_corx_random_",iter,"_FINAL.txt",sep = ""))


#percentage explaied varaince is calculated by multiplying last element (ntree) in rsq  with 100
rf3_v1 = rf3_1$rsq[20000]*100


for (i in nS){#SNPs halved at each iteration of Coarse Sweep
  
  ntrees = 10000

  
  imp_rf3_1<-data.frame(importance(rf3_1,type=1))
  colnames(imp_rf3_1)<-c("importance")
  rf3_ord1 = order(imp_rf3_1[,1],decreasing = T)
  rf3_corr_1<-as.character(rownames(imp_rf3_1)[rf3_ord1[1:i]])
  
  if(i==120){
    
    imp_all_1 = as.data.frame(importance(rf3_1))
    
    
    imp_all_1$imp_score = imp_all_1[,1]/abs(min( imp_all_1[,1]))
    #Write top 120 SNPs importance
    write.table(imp_all_1,paste("RANDOM_RF_Files/importance_120_imp_RF3_1_SNPs_FL_corx_random_",iter,"_FINAL.txt",sep = ""))
    
    #Write top 120 SNPs for Fine-sweep Random Forest analysis
    write.table(rf3_corr_1,file = paste("RANDOM_RF_Files/Top120SNPs_FL_corx_random_",iter,"_FINAL.txt",sep = ""))
  }
  corr_rf3_x0<-corr_x0[,rf3_corr_1]#subset SNPs
  #fitting random forest model, trait residuals as responce and this snps subset as feeatures
  rf3_1 = randomForest(x = corr_rf3_x0, y = FL_corx_random, importance=TRUE ,proximity=TRUE, ntree=ntrees)

  
  #percentage explaied varaince is calculated by multiplying last element (ntree) in rsq  with 100
  rf3_v1 = c(rf3_v1,rf3_1$rsq[ntrees]*100)
  
  
}

out20000<-cbind(out20000,rf3_v1)

#Write RF Coarse-sweep, percentage explaied varaince of each subset
write.table(out20000,file = paste("RANDOM_RF_Files/RF_20000trees_FL_corx_random_",iter,"_FINAL.txt",sep = ""))
}
#The following file is needed to plot the results as in the original publication
write.table(out20000,file = "RANDOM_RF_Files/RF_20000trees_FL_corx_random_allnIterations_FINAL.txt",sep ="\t")
